### Module 2

library(lme4)
?glmer

library(geepack)
?geeglm

summ.fxn <- function(x) {
	nobs = length(x[!is.na(x)])
	mean = mean(x[!is.na(x)])
	sd = sd(x[!is.na(x)])
	min = min(x[!is.na(x)])
	max = max(x[!is.na(x)])

	return(c(nobs=round(nobs,0), mean=mean, sd=sd, min=min, max=max))
}


### Case Study: Postnatal Depression
rm(list=ls())
load('depress.RData')

attach(depress)

summ.fxn(dep0[group=="placebo"])
summ.fxn(dep1[group=="placebo"])
summ.fxn(dep2[group=="placebo"])
summ.fxn(dep3[group=="placebo"])
summ.fxn(dep4[group=="placebo"])
summ.fxn(dep5[group=="placebo"])
summ.fxn(dep6[group=="placebo"])

summ.fxn(dep0[group=="estrogen"])
summ.fxn(dep1[group=="estrogen"])
summ.fxn(dep2[group=="estrogen"])
summ.fxn(dep3[group=="estrogen"])
summ.fxn(dep4[group=="estrogen"])
summ.fxn(dep5[group=="estrogen"])
summ.fxn(dep6[group=="estrogen"])

# pairwise correlation plots:
pairs(~dep0+dep1+dep2+dep3+dep4+dep5+dep6)

subp=(!is.na(dep6) & !is.na(dep0) & group=="placebo")
t.test(dep0[subp],dep6[subp], paired=T)

sube=(!is.na(dep6) & !is.na(dep0) & group=="estrogen")
t.test(dep0[sube],dep6[sube], paired=T)

t.test(I(dep6-dep0) ~ group)

## calculate correlation across measurements (also var-cov matrix)
## note: Stata "corr" command uses participants measured at all times; 
##	    code below only requires pairwise-complete observations
ntime <- 7
rmat <- with(depress, cbind(dep0, dep1, dep2, dep3, dep4, dep5, dep6))
	
vmat <- nmat <- covmatj <- covmatk <- matrix(0, nrow=ntime, ncol=ntime)
rownames(vmat) <- colnames(vmat) <- 1:ntime
rownames(nmat) <- colnames(nmat) <- 1:ntime

for(j in 1:ntime){
    for(k in 1:ntime){
        nmat[j,k] <- sum(!is.na(rmat[,j]*rmat[,k]))
 	  vmat[j,k] <- cov(rmat[,j],rmat[,k],use="pairwise.complete.obs")
	 if(nmat[j,k]>0) covmatj[j,k] <- cov(rmat[,j][!is.na(rmat[,j]*rmat[,k])],rmat[,j][!is.na(rmat[,j]*rmat[,k])],use="pairwise.complete.obs")
	 if(nmat[j,k]>0) covmatk[j,k] <- cov(rmat[,k][!is.na(rmat[,j]*rmat[,k])],rmat[,k][!is.na(rmat[,j]*rmat[,k])],use="pairwise.complete.obs")
	}}

cmat <- vmat/sqrt(covmatj)/sqrt(covmatk)

round(vmat,3)
round(cmat, 2)
nmat
		


# change data to long form
detach(depress)
depresslong <- reshape(data=depress, varying = c("dep0", "dep1", "dep2", "dep3", "dep4", "dep5", "dep6"), 
	v.names="dep", timevar="visit", idvar="subj", times = 0:6, 
	new.row.names = 1:427, direction="long")
depresslsort <- depresslong[order(depresslong$subj),]

attach(depresslsort)

# plot depression scores vs visit, by treatment group
par(bty="n", mar=c(5, 5, 2, 2) + 0.1, cex.axis=1.5, cex.lab=1.5, cex.main=1.5)
plot(jitter(visit[group=="placebo"]), dep[group=="placebo"], 
	pch=21, col="blue", bg="blue", xlab="Visit", ylab="Depression Score", ylim=c(0,30))
points(jitter(visit[group=="estrogen"]), dep[group=="estrogen"], 
	pch=21, col="darkgreen", bg="darkgreen", xlab="Visit", ylab="Depression Score", ylim=c(0,30))
lines(lowess(visit[!is.na(dep) & group=="placebo"], 
	dep[!is.na(dep) & group=="placebo"]), lwd=5, col="blue")
lines(lowess(visit[!is.na(dep) & group=="estrogen"], 
	dep[!is.na(dep) & group=="estrogen"]), lwd=5, col="darkgreen")
legend(3,30, legend=c("Placebo", "Estrogen"), pch=c(21,21), col=c("blue", "darkgreen"), pt.bg=c("blue", "darkgreen"), lty=c(1,1), bty="n", cex=1.5, lwd=2)


boxplot(dep~visit, data=depresslsort)


# plot depression scores vs visit, by individual
par(bty="n", mar=c(5, 5, 2, 2) + 0.1, cex.axis=1.5, cex.lab=1.5, cex.main=1.5)
plot(visit, dep, xlab="Visit", ylab="Depression Score", ylim=c(0,30), type="n", main="")
for(i in 1:61){
	lines(visit[subj==i], dep[subj==i], lwd=2, col="dodgerblue4")}


# GEE
m1 <- geeglm(dep~group+visit, id=subj, data=depresslsort, corstr="independence")
m1.1 <- geeglm(dep~group+visit, id=subj, data=subset(depresslsort, visit!=0), corstr="independence")

m2 <- geeglm(dep~group+visit, id=subj, data=depresslsort, corstr="ar1")

summary(m1)
summary(m2, correlation=T)

genCor <- function(maxClustSize, alpha, corstr){
	if(corstr=="independence"){#indep
		return(as.matrix(Diagonal(x=rep(1,maxClustSize))))	
	}
	if(corstr=="exchangeable"){#exch
			return(alpha^(!outer(1:maxClustSize,1:maxClustSize,"==")))
	}
	if(corstr=="ar1"){#ar1
			return(alpha^abs(outer(1:maxClustSize,1:maxClustSize,"-")))
	}
	if (corstr == "unstructured") {
		m <- as.matrix(Diagonal(x=rep(1,maxClustSize)))
		m[lower.tri(m)] <- alpha
		m <- t(m)
		m[lower.tri(m)] <- alpha
	}
	stop("Unrecognized correlation structure, should be one of 'independence', 'exchangeable', 'unstructured', or 'ar1'")
}

genCor(maxClustSize=max(summary(m2)$clusz),alpha=summary(m2)$corr[1,1], corstr="ar1")


# categorical visit term
m3 <- geeglm(dep~group+factor(visit), id=subj, data=depresslsort, corstr="ar1")
m0 <- geeglm(dep~group, id=subj, data=depresslsort, corstr="ar1")
summary(m3)
anova(m0,m3)


# interaction between visit and treatment group
m4 <- geeglm(dep~group*visit, id=subj, data=depresslsort, corstr="ar1")
summary(m4)

# calculate linear combination
m4$coef["visit"]+m4$coef["groupestrogen:visit"]
# and its standard error
sqrt(summary(m4)$cov.scaled[3,3] + summary(m4)$cov.scaled[4,4] + 2*summary(m4)$cov.scaled[3,4]) 

### alternate method using function in 'car' package:
library(car)
depresslsort$groupvisit=with(depresslsort,(as.numeric(group)-1)*as.numeric(visit))
m4=geeglm(dep~group+visit+groupvisit, id=subj, data=depresslsort, corstr="ar1")
m4.1 <- geeglm(dep~visit + group:visit, id=subj, data=depresslsort, corstr="ar1")
deltaMethod(coef(m4),"visit+groupvisit",vcov.=summary(m4)$cov.scaled)




### Case Study: ICHS

rm(list=ls())
load('ichs.RData')

## Exploratory data analysis

# Categorize by year of age
ichs <- within(ichs, year <- floor((age+36)/12))

# Cross-tabulate vitamin A deficiency and respiratory infection
with(ichs, table(year,xerop,infection))
    		
# Selected individual trajectories
ichsIDs <- c(121013,121113,121114,121140,121414,122214,122315,152150,141517)

par(bty="n", mar=c(4.5, 4.5, 2, 2) + 0.1, cex.axis=1.5, cex.lab=1.5, mfrow=c(3,3))
for(i in 1:length(ichsIDs)) {
	with(ichs, plot(age+36, infection, type="n", xlab="Age (years)", ylab="", ylim=c(0, 3), axes=FALSE))
	title(main=paste("subject",ichsIDs[i],sep=""))
	axis(1, at=c(12,24,36,48,60,72,84), labels=1:7)
	axis(2, at=c(0,1,2,3), labels=c("N", "Y", "N", "Y"), las=1)	
	mtext("Xerop",side=2, las=1, at=0.5)
	mtext("Infection",side=2, las=1, at=2.5)
	with(ichs, lines(age[id==ichsIDs[i]]+36, infection[id==ichsIDs[i]]+2, col="firebrick4", lwd=3))
	with(ichs, lines(age[id==ichsIDs[i]]+36, xerop[id==ichsIDs[i]], col="skyblue4", lwd=3))
}

# Probabilities of infection and vitamin A deficiency by age
infectionbyyear.xerop <- with(ichs, split(infection[xerop==1],as.factor(age[xerop==1])))
meaninfbyyear.xerop <- lapply(infectionbyyear.xerop,mean)
time.xerop <- with(ichs, sort(unique(age[xerop==1])))

infectionbyyear.noxerop <- with(ichs, split(infection[xerop==0],as.factor(age[xerop==0])))
meaninfbyyear.noxerop <- lapply(infectionbyyear.noxerop,mean)
time.noxerop <- with(ichs, sort(unique(age[xerop==0])))

par(bty="n", mar=c(4.5, 4.5, 2, 2) + 0.1, cex.axis=1.5, cex.lab=1.5)
plot(time.noxerop+36, meaninfbyyear.noxerop, type="n", xlab="Age (years)", ylab="Percent with Respiratory Infection",  xlim=c(0,86),ylim=c(0, 1), axes=FALSE)
axis(1, at=c(0,12,24,36,48,60,72,84), labels=0:7)
axis(2, c(0,0.2,0.4,0.6,0.8,1),las=1)
points(time.noxerop+36, meaninfbyyear.noxerop, col="firebrick4", pch=1, cex=2)	
points(time.xerop+36, meaninfbyyear.xerop, col="skyblue4", pch=16,cex=2)
legend(48,0.75, legend=c("No Xerop", "Xerop"), pch=c(1,16), col=c("firebrick4", "skyblue4"), bty="n", cex=1.5)

# Probabilities of infection and vitamin A deficiency by age in years
infectionbyyear.xerop <- with(ichs, split(infection[xerop==1],as.factor(year[xerop==1])))
meaninfbyyear.xerop <- lapply(infectionbyyear.xerop,mean)
year.xerop <- with(ichs, sort(unique(year[xerop==1])))

infectionbyyear.noxerop <- with(ichs, split(infection[xerop==0],as.factor(year[xerop==0])))
meaninfbyyear.noxerop <- lapply(infectionbyyear.noxerop,mean)
year.noxerop <- with(ichs, sort(unique(year[xerop==0])))

par(bty="n", mar=c(4.5, 4.5, 2, 2) + 0.1, cex.axis=1.5, cex.lab=1.5)
plot(year.noxerop, meaninfbyyear.noxerop, type="n", xlab="Age (years)", ylab="Percent with Respiratory Infection",  ylim=c(0, 1), axes=FALSE)
axis(1, at=0:7, labels=0:7)
axis(2, c(0,0.2,0.4,0.6,0.8,1),las=1)
points(year.noxerop, meaninfbyyear.noxerop, col="firebrick4", pch=1, cex=2)	
points(year.xerop, meaninfbyyear.xerop, col="skyblue4", pch=16,cex=2)
legend(4,0.75, legend=c("No Xerop", "Xerop"), pch=c(1,16), col=c("firebrick4", "skyblue4"), bty="n", cex=1.5)

# Logistic regression model (ignoring correlation)
model <- glm(infection ~ xerop + age + gender + hfora + cost + sint, 
                data=ichs, family="binomial")
summary(model)
exp(model$coef[2])
exp(model$coef[2]+qnorm(0.975)*summary(model)$coef[2,2])
exp(model$coef[2]-qnorm(0.975)*summary(model)$coef[2,2])

# Variance, covariance, correlations
ichswide <- reshape(data=with(ichs, data.frame(infection, id, time)), v.names="infection", timevar="time", idvar="id", direction="wide")

rmat <- with(ichswide, cbind(infection.1, infection.2, infection.3, 
					infection.4, infection.5, infection.6))
	
cor.mat <- or.mat <- n.mat <- matrix(0, nrow=6, ncol=6)
for(i in 1:6){
  for(j in i:6){
    temp <- na.omit(rmat[,c(i,j)])
    month.i <- temp[,1]
    month.j <- temp[,2]
    cor.mat[i,j] <- cor(month.i, month.j)
    table.ij <- table(month.i, month.j)
    or.mat[i,j] <- (table.ij[2,2]*table.ij[1,1])/(table.ij[1,2]*table.ij[2,1])
    n.mat[i,j] <- length(month.i)}}
round(cor.mat, 2)
round(or.mat, 2)
n.mat
	
round(colMeans(rmat,na.rm=T),2) 

## Generalized estimating equations

rm(list=ls())
load('ichs.RData')

m1 <- geeglm(infection ~ xerop + age + gender + hfora + cost + sint, 
             id=id, data=ichs, family="binomial", corstr="independence")
             
m2 <- geeglm(infection ~ xerop + age + gender + hfora + cost + sint, 
             id=id, data=ichs, family="binomial", corstr="exchangeable")
             
m3 <- geeglm(infection ~ xerop + age + gender + hfora + cost + sint,
              id=id, data=ichs, family="binomial", corstr="ar1")

exp(m1$coef[2])
exp(m1$coef[2]+qnorm(0.975)*summary(m1)$coef[2,2])
exp(m1$coef[2]-qnorm(0.975)*summary(m1)$coef[2,2])

exp(m2$coef[2])
exp(m2$coef[2]+qnorm(0.975)*summary(m2)$coef[2,2])
exp(m2$coef[2]-qnorm(0.975)*summary(m2)$coef[2,2])

summary(m2)$geese$correlation[1]

exp(m3$coef[2])
exp(m3$coef[2]+qnorm(0.975)*summary(m3)$coef[2,2])
exp(m3$coef[2]-qnorm(0.975)*summary(m3)$coef[2,2])

summary(m3)$geese$correlation[1]
summary(m3)$geese$correlation[1]^2
summary(m3)$geese$correlation[1]^3
summary(m3)$geese$correlation[1]^4
summary(m3)$geese$correlation[1]^5

## Generalized linear mixed-effects models

rm(list=ls())
load('ichs.RData')


m_ri <- glmer(infection ~ (1 | id) + factor(xerop) + age + factor(gender) + hfora + cost + sint,
	 family=binomial, data=ichs, nAGQ=7) #note: nAGQ=7 is used in Stata
summary(m_ri)

expit <- function(x){exp(x)/(1+exp(x))}

expit(fixef(m_ri)[1])
expit(fixef(m_ri)[1]-1.96*sqrt(VarCorr(m_ri)$id[[1]]))
expit(fixef(m_ri)[1]+1.96*sqrt(VarCorr(m_ri)$id[[1]]))




