* Module 2

* Case Study: Postnatal Depression
use "depress.dta", clear

* Exploratory Data Analysis
sort group
by group: summarize dep0 dep1 dep2 dep3 dep4 dep5 dep6

graph matrix dep0 dep1 dep2 dep3 dep4 dep5 dep6, half

sort group
by group: ttest dep0==dep6

corr dep0 dep1 dep2 dep3 dep4 dep5 dep6

gen diff=dep6-dep0
ttest diff, by(group) unequal

* change data to long form
reshape long dep, i(subj) j(visit)

xtline dep, i(subj) t(visit) overlay legend(off)
     xlab(0(1)6) xtitle("Visit") ytitle("Depression Score")

separate dep, by(group)
graph twoway (scatter dep0 visit, jitter(10) mcolor(blue)) (scatter dep1 visit, jitter(10) mcolor(green)) ///
     (lowess dep0 visit, lcolor(blue)) (lowess dep1 visit, lcolor(green)), ///
     xtitle(Visit) ytitle(Depression Score) ///
     legend(order(1 "Placebo" 2 "Estrogen"))

* GEE
xtset subj visit
xtgee dep visit i.group, corr(ind) robust
xtgee dep visit i.group if visit!=0, corr(ind) robust

xtgee dep visit i.group, corr(ar1) robust
estat wcorr

xtgee dep i.group i.visit, corr(ar1) robust
testparm i.visit

xtgee dep c.visit##i.group, corr(ar1) robust
lincom visit + 1.group#c.visit

xtgee dep visit c.visit#i.group, corr(ar1) robust


* Case Study: ICHS
* Exploratory data analysis

use ichs.dta, replace

list id age time infection xerop gender hfora cost sint

* Categorize by year of age
gen year=floor((age+36)/12)

* Cross-tabulate vitamin A deficiency and respiratory infection
bysort xerop: tab year infection


* Declare the dataset to be "panel" data, grouped by id
* with time variable time 
xtset id time

* Individual trajectories
xtline infection if id<121141, legend(off)
xtline xerop if id<121141, legend(off)

* Probabilities of infection and vitamin A deficiency
bysort xerop: summarize infection if time==1
bysort xerop: summarize infection if time==2
bysort xerop: summarize infection if time==3
bysort xerop: summarize infection if time==4
bysort xerop: summarize infection if time==5
bysort xerop: summarize infection if time==6

gen ageyear=floor((age+36)/12)
bysort xerop: summarize infection if ageyear==0
bysort xerop: summarize infection if ageyear==1
bysort xerop: summarize infection if ageyear==2
bysort xerop: summarize infection if ageyear==3
bysort xerop: summarize infection if ageyear==4
bysort xerop: summarize infection if ageyear==5
bysort xerop: summarize infection if ageyear==6

* Covariance and correlation
drop age ageyear cost sint hfora year
reshape wide infection xerop, i(id) j(time)
corr infection1 infection2 infection3 infection4 infection5 infection6
corr infection1 infection2 infection3 infection4 infection5 infection6, cov

* Generalized estimating equations

use ichs.dta, replace

* Declare the dataset to be "panel" data, grouped by id
* with time variable time
xtset id time

* Fit a GEE model with exchangeable working correlation
xtgee infection xerop age gender hfora cost sint, family(binomial) link(logit) corr(exch) robust
estat wcorr

* Generalized linear mixed-effects models

* Fit a model with random intercepts
help melogit
melogit infection i.xerop age i.gender hfora cost sint || id:

* Obtain predicted probabilities of infection, 
* setting the random effects to 0
margins i.xerop, predict(mu fixed)




